@@ -13,10 +13,10 @@
 #include <clutter/clutter.h>
 #include <fcntl.h> //we check some files
 
-
+#include "singinggame.h"
 #include "themeselection.h"
 #include "file_helper.h"
-#include "string.h"
+#include <string.h>
 
 #include "gui.h" //for chum_states
 #include "chum.h" //for chum_states
@@ -24,27 +24,71 @@
 #include "../meam/jammo-playing-track.h"
 
 
+/*
+It suppose filenames are formatted in this way (will seg fault if not).
+folder/+theme+[123]+[ae]+date.
+example:+fantasy+2+e+2010.03.11_00.03.52.txt
+theme_name = city / animal / fantasy          (this is checked, affects icon graphics)
+variation = 1 / 2 / 3                         (not critical, user will get empty set of samplebuttons, if use non-valid variation)
+mode (easy/advanced) = e /a                   (if unclear, interpret advanced, because there can be two tracks)
+
+But this is callback-function for icon and we do not make icon if it doesn't correspond with these rules.
+ (We do not want broken icons and error messages)
+
+*/
 static gboolean cupboard_icon_clicked(TangleButton *tanglebutton, gpointer data){
 	printf("Load icon pressed\n");
 	ClutterActor* actor = jammo_get_actor_by_id("cupboard-view");
 	clutter_actor_hide(actor);
 
-	themeselection_start_theme("fantasy");
+	//We need copy because strtok overwrite.
+	gchar *filename; 
+	filename = g_strdup_printf("%s",(gchar*)data);
+	printf("filename '%s'\n",filename); // 
+
+
+	strtok(filename, "+"); //this first is directory path. No use in here.
+
+	char* theme_name =  strtok(NULL, "+");
+	char* variation_name = strtok(NULL, "+");
+	char* easy  = strtok(NULL, "+");
 	
-	//We must give time to start_theme (for some unknown reason)
-	g_timeout_add(100,themeselection_load,(char*)data);
-	//themeselection_load();
+	if (strncmp(easy,"e",1)==0)
+		chum_set_easy_game(TRUE); //e=easy
+	else //if (strncmp(easy,"a",1)==0) //it is safer to use advanced if this is not clear
+		chum_set_easy_game(FALSE); //a=advanced
+
+	themeselection_start_theme(theme_name,atoi(variation_name),jammo_get_actor_by_id("cupboard-view"));
+
+	//This is hack.  FIXME: for some reason we can't just call this.
+	g_timeout_add(100,themeselection_load,data);
+
+
 	return FALSE;
 }
 
-static JammoSequencer* sequencer;
-static void play_sequencer(TangleActor *actor, gpointer user_data) {
-//jammo_sequencer_play(JAMMO_SEQUENCER(user_data));
-jammo_sequencer_play(sequencer);
+
+
+
+static void play_sequencer (TangleActor *actor, ClutterActorBox *arg1, gpointer user_data){
+	singinggame_start_sequencer(JAMMO_SEQUENCER(user_data));
 }
 
 
 
+static void on_sequencer_stopped(JammoSequencer* sequencer_old, gpointer none) {
+	//Hide all
+	ClutterActor*	mainview = jammo_get_actor_by_id("main-views-widget"); //No drawing on stage anymore, use this.
+	clutter_container_foreach(CLUTTER_CONTAINER(mainview), tangle_actor_hide_animated, NULL);
+	
+	//Show cupboard
+	ClutterActor* view = jammo_get_actor_by_id("cupboard-view");
+	clutter_actor_show(view);
+
+	//And show mentor again
+	tangle_actor_show(TANGLE_ACTOR(jammo_mentor_get_default()));
+}
+
 /*
 
 */
@@ -63,14 +107,9 @@ static gboolean cupboard_song_icon_clicked(TangleButton *tanglebutton, gpointer
 	jammo_mentor_shut_up(jammo_mentor_get_default());
 	tangle_actor_hide_animated(TANGLE_ACTOR(jammo_mentor_get_default()));
 
-
-	//add tracks to sequencer
-	if (sequencer) {
-		jammo_sequencer_stop(sequencer);
-		g_object_unref(sequencer);
-	}
-	sequencer = jammo_sequencer_new();
-
+	//make new sequencer and add tracks to it
+	JammoSequencer* sequencer = jammo_sequencer_new();
+	g_signal_connect(sequencer, "stopped", G_CALLBACK(on_sequencer_stopped), NULL);
 
 	//TODO: maybe we not should use currently selected_language, but save it too.
 	gchar *filename;
@@ -121,28 +160,20 @@ static gboolean cupboard_song_icon_clicked(TangleButton *tanglebutton, gpointer
 	return FALSE;
 }
 
-//Scrolling area for icons
-static ClutterActor* scrolling_actor = NULL;
+
 
 /*
 This will also check and load icons for savings.
 */
 void cupboard_start_cupboard(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
-	//Stop if there are sequencer playing
-	if (sequencer) {
-		jammo_sequencer_stop(sequencer);
-		g_object_unref(sequencer);
-	}
-	
-	if (scrolling_actor){
-	clutter_actor_destroy(CLUTTER_ACTOR(scrolling_actor));
-	}
+	//Scrolling area for icons
+	ClutterActor* scrolling_actor = NULL;
 	
 	ClutterActor* container_for_saved_icons;
 	container_for_saved_icons = tangle_widget_new();
 	clutter_actor_set_size(container_for_saved_icons, 210.0, 1800.0);
 
-	//Load compositions TODO. This is just test-version with one icon.
+	//Load compositions
 	gchar *filepath1;
 	filepath1 = COMPOSITIONS_DIRECTORY;
 	
@@ -152,11 +183,43 @@ void cupboard_start_cupboard(TangleAction *action, GObject *source, const gchar
 	GList* l;
 	for (l = files; l; l = l->next)
 		{
-		printf("readed '%s'\n",(char*)l->data);
-		ClutterActor* icon;
-		//if (strncmp(name,"fant",4)==0) TODO //Use correct icon
-		icon = tangle_button_new_with_background_actors(clutter_texture_new_from_file("/opt/jammo/themes/fantasy/110/ghost.png", NULL), clutter_texture_new_from_file("/opt/jammo/themes/fantasy/110/ghost.png", NULL));
+		//printf("readed '%s'\n",(char*)l->data);
+	
+		//We need copy because strtok overwrite.
+		gchar *filename; 
+		filename = g_strdup_printf("%s",(gchar*)l->data);
+		//printf("filename '%s'\n",filename); 
 
+		strtok(filename, "+"); //this is directory path. No use in here.
+
+		char* theme_name =  strtok(NULL, "+");
+		char* variation_name = strtok(NULL, "+");
+		char* easy  = strtok(NULL, "+");
+	
+		ClutterActor* icon;
+		gchar* icon_filename;
+		if (theme_name == NULL || variation_name == NULL || easy == NULL) //This means, we couldn't tokenizer it -> do not show it at all
+			break;
+		
+		if (strncmp(theme_name,"fantasy",7)==0) 
+		{
+			icon_filename = strncmp(easy,"e",1)==0 ? "/opt/jammo/themes/fantasy/140/donkey.png" : "/opt/jammo/themes/fantasy/90/unicorn.png";
+		}
+		else if (strncmp(theme_name,"city",4)==0) 
+		{
+			icon_filename = strncmp(easy,"e",1)==0 ? "/opt/jammo/themes/city/140/bicycle.png" : "/opt/jammo/themes/city/140/motorcycle.png";
+		}
+		else if (strncmp(theme_name,"animal",6)==0) 
+		{
+			icon_filename = strncmp(easy,"e",1)==0 ? "/opt/jammo/themes/animal/90/monkey.png" : "/opt/jammo/themes/animal/90/goril.png";
+		}
+		else //now we have hardcoded theme names TODO.  Not correct theme name -> do not show it at all
+		{
+			break;
+		}
+
+
+		icon = tangle_button_new_with_background_actors(clutter_texture_new_from_file(icon_filename, NULL), clutter_texture_new_from_file(icon_filename, NULL));
 		g_object_set(icon,"depth-position", -1, "width", 60.0, "height", 60.0, "x", 50.0,  "y", 20+counter1*95.0, NULL);
 		
 		clutter_container_add(CLUTTER_CONTAINER(container_for_saved_icons), icon, NULL);
@@ -174,22 +237,22 @@ void cupboard_start_cupboard(TangleAction *action, GObject *source, const gchar
 	int counter2=0;
 	for (l = files; l; l = l->next)
 		{
-		printf("readed '%s'\n",(char*)l->data);
+		//printf("readed '%s'\n",(char*)l->data);
 		ClutterActor* icon;
 		//int length_of_prefix= strlen(filepath1);
 
 		char* name = (char*)l->data;
-		printf("name = '%s'\n",name);
+		//printf("name = '%s'\n",name);
 		char* name1 = &name[strlen(filepath1)+1];
-		printf("name1 = '%s'\n",name1);
+		//printf("name1 = '%s'\n",name1);
 	
 		char* name2 = malloc (20);
 		strncpy( name2, name1,strlen(name1)-4);
 		name2[strlen(name1)-4]='\0';
-		printf("name2 = '%s'\n",name2);
+		//printf("name2 = '%s'\n",name2);
 
 		gchar* icon_filename=g_strdup_printf("%s/songs/%s/icon.png",DATA_DIR,name2);
-		printf("looking for '%s'\n",icon_filename);
+		//printf("looking for '%s'\n",icon_filename);
 		
 		icon = tangle_button_new_with_background_actors(clutter_texture_new_from_file(icon_filename, NULL), clutter_texture_new_from_file(icon_filename, NULL));
 		g_free(icon_filename);
