@@ -212,6 +212,9 @@ int main(int argc, char** argv) {
 	clutter_container_add_actor(CLUTTER_CONTAINER(stage), actor);
 	
 	clutter_actor_show(stage);
+
+	//do not show mentor on startup
+	clutter_actor_hide(CLUTTER_ACTOR(jammo_mentor_get_default()));
 	
  if (argc==2 && strcmp(argv[1],"editor")==0) { 
 	start_midi_editor();
