@@ -20,14 +20,12 @@
 #include "jammo-mentor.h"
 #include "../meam/jammo-playing-track.h"
 
-JammoRecordingTrack* recording_track;
-
-static JammoSequencer* sequencer;
+static JammoSequencer* singinggame_sequencer = NULL; //used for stopping
 
 
-ClutterActor *contour_cursor;
-ClutterActor *contour_trace;
-ClutterActor *singinggame_display_freq;
+static ClutterActor *contour_cursor;
+static ClutterActor *contour_trace;
+static ClutterActor *singinggame_display_freq;
 
 
 /*
@@ -100,28 +98,119 @@ static void on_pitch_detected(JammoRecordingTrack* track, gfloat frequency, gpoi
 
 
 
-
-
-
-
-static void singingame_start_sequencer(JammoMentor* mentor, const gchar* speech, gpointer user_data){
-tangle_actor_hide_animated(TANGLE_ACTOR(jammo_mentor_get_default()));
-jammo_sequencer_play(sequencer);
+/*
+This function is globally visible.
+Whenever 'cross-icon' is pressed, this sequencer is stopped.
+*/
+void singinggame_start_sequencer(JammoSequencer* sequencer) {
+	singinggame_sequencer = sequencer;
+	jammo_sequencer_play(sequencer);
 }
 
 
-static void song_view_showed(TangleActor *actor, ClutterActorBox *arg1, ClutterActorBox *arg2, gpointer user_data){
+static void start_sequencer(JammoMentor* mentor, const gchar* speech, gpointer sequencer){
+	clutter_actor_hide(TANGLE_ACTOR(jammo_mentor_get_default()));
+	singinggame_start_sequencer(JAMMO_SEQUENCER(sequencer));
+}
+
+
+static void song_view_showed(TangleActor *actor, ClutterActorBox *arg1, ClutterActorBox *arg2, gpointer sequencer){
 	printf("song_view_showed\n");
 
 	gchar *filepath1;
 	filepath1 =  g_strdup_printf("/opt/jammo/mentor_speech/mentor_song_%s_fi.ogg", chum_get_current_song());
-	printf("pass to mentor: '%s' \n",filepath1);
+	//printf("pass to mentor: '%s' \n",filepath1);
 
-	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), filepath1, singingame_start_sequencer, NULL);
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), filepath1, start_sequencer, sequencer);
 	jammo_mentor_set_idle_speech(jammo_mentor_get_default(), "");
 }
 
 
+static void on_sequencer_stopped_playback(JammoSequencer* sequencer_old, gpointer none) {
+	printf("Sequencer stopped_playback\n");
+
+	//Hide all
+	ClutterActor*	mainview = jammo_get_actor_by_id("main-views-widget"); //No drawing on stage anymore, use this.
+	clutter_container_foreach(CLUTTER_CONTAINER(mainview), tangle_actor_hide_animated, NULL);
+	
+	//Go back to songselection
+	ClutterActor* view = jammo_get_actor_by_id("songselection-view");
+	clutter_actor_show(view);
+
+	//And show mentor again
+	tangle_actor_show(TANGLE_ACTOR(jammo_mentor_get_default()));
+
+
+	//Automatic saving. THIS MUST BE DOCUMENTED.
+
+	//Saving to cupboard and to log.
+	gchar* cmd;
+
+
+	//Copy that recorded song to 'singings' and name it.
+	cmd = g_strdup_printf("cp %s/last_recorded.wav %s/%s.wav", JAMMO_DIRECTORY, SINGINGS_DIRECTORY,chum_get_current_song() );
+	if (system(cmd)) 
+		printf("Error, can't call '%s'\n",cmd);
+
+
+	//And make it permanent copy for carer/researcher. Filenames look like this: log_singing_2010.02.22_21.16.27.wav
+	cmd = g_strdup_printf("cp %s/last_recorded.wav %s/log_singing_`date +%%Y.%%m.%%d_%%H.%%M.%%S`.wav",JAMMO_DIRECTORY,JAMMO_DIRECTORY);
+	if (system(cmd)) 
+		printf("Error, can't call '%s'\n",cmd);
+
+	g_free(cmd);
+}
+
+
+
+static void on_sequencer_stopped_recording(JammoSequencer* sequencer_old, gpointer none) {
+	printf("Sequencer stopped_recording\n");
+
+	
+	#ifndef OFFICIAL_RELEASE
+	clutter_actor_hide(contour_cursor);
+	clutter_actor_hide(contour_trace);
+	clutter_actor_hide(singinggame_display_freq);
+	contour_counter=LEFT_MARGINAL;
+	#endif
+
+
+	JammoSample* sample1;
+	JammoSample* sample2;
+	JammoEditingTrack* backing_track;
+	JammoEditingTrack* recorded_track;
+	JammoSequencer* sequencer = jammo_sequencer_new();
+	
+	g_signal_connect(sequencer, "stopped", G_CALLBACK(on_sequencer_stopped_playback), NULL);
+
+	backing_track = jammo_editing_track_new();
+	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(backing_track));
+
+	gchar *filename = g_strdup_printf("%s/songs/%s/comping%s.ogg", DATA_DIR,chum_get_current_song(),chum_get_selected_language());
+	int fd = open(filename,O_RDONLY);
+	if (fd == -1) //File not found
+		{
+		filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,chum_get_current_song()); //use without language postfix
+		}
+	sample1 = jammo_sample_new_from_file(filename);
+	jammo_editing_track_add_sample(backing_track, sample1, 0);
+	g_free(filename);
+
+
+	//for recorded singing
+	recorded_track = jammo_editing_track_new(); 
+	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(recorded_track));
+	gchar* filename2 = g_strdup_printf("%s/last_recorded.wav",JAMMO_DIRECTORY);
+	sample2 = jammo_sample_new_from_file(filename2);
+	g_free(filename2);
+
+	jammo_editing_track_add_sample(recorded_track, sample2, 0);
+	chum_set_state_of_chum(SCREEN_PLAY_RECORDED);
+
+	jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "/opt/jammo/mentor_speech/mentor_welldone_fi.ogg", start_sequencer, sequencer);
+}
+
+
 /*
 trigger="clicked"
 Source= TangleButton.
@@ -135,10 +224,9 @@ void singinggame_song_click(TangleAction *action, GObject *source, const gchar *
 	
 	chum_set_current_song(song_name);
 	
-	if (sequencer) {
-		g_object_unref(sequencer);
-	}
-	sequencer = jammo_sequencer_new();
+
+	JammoSequencer* sequencer = jammo_sequencer_new();
+	g_signal_connect(sequencer, "stopped", G_CALLBACK(on_sequencer_stopped_recording),NULL);
 
 	gchar *filename;
 	int fd;
@@ -168,6 +256,7 @@ void singinggame_song_click(TangleAction *action, GObject *source, const gchar *
 	jammo_sequencer_add_track(sequencer, JAMMO_TRACK(playing_track));
 
 	gchar* recording_filename = g_strdup_printf("%s/last_recorded.wav",JAMMO_DIRECTORY);
+	JammoRecordingTrack* recording_track;
 	//Do not use pitch_detect on OFFICIAL_RELEASE (not robust enough)
 	#ifdef OFFICIAL_RELEASE
 	recording_track = jammo_recording_track_new(recording_filename);
@@ -189,8 +278,8 @@ void singinggame_song_click(TangleAction *action, GObject *source, const gchar *
 	g_free(name_of_view);
 
 
-	//g_signal_connect(view, "transition-completed", G_CALLBACK(song_view_showed), NULL); FIXME: this needs modifications to json
-	song_view_showed(NULL,NULL,NULL,NULL);
+	//g_signal_connect(view, "transition-completed", G_CALLBACK(song_view_showed), sequencer); FIXME: this needs modifications to json
+	song_view_showed(NULL,NULL,NULL,sequencer);
 
 	//Do not use pitch_detect on OFFICIAL_RELEASE (not robust enough)
 	#ifndef OFFICIAL_RELEASE
@@ -252,129 +341,24 @@ void singinggame_song_click(TangleAction *action, GObject *source, const gchar *
 
 }
 
-void singinggame_cross_click(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
-	ClutterActor* actor =CLUTTER_ACTOR(source);
 
+
+void singinggame_cross_click(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
+/*
 	const gchar* id_of_actor;
 	id_of_actor = clutter_get_script_id(source);
-			
+
 	printf("cross_clicked. get_id: '%s' and parent '%s'\n",id_of_actor,clutter_get_script_id(G_OBJECT(clutter_actor_get_parent(CLUTTER_ACTOR(source)))));
+*/
 
-
-	if (chum_get_state_of_chum()==SCREEN_SONG_RECORDING)
-		{
-		printf("chum_get_state_of_chum()==SCREEN_SONG_RECORDING\n");
-
-		#ifndef OFFICIAL_RELEASE
-		clutter_actor_hide(contour_cursor);
-		clutter_actor_hide(contour_trace);
-		clutter_actor_hide(singinggame_display_freq);
-		contour_counter=LEFT_MARGINAL;
-		#endif
-
-
-		JammoSample* sample1;
-		JammoSample* sample2;
-		JammoEditingTrack* backing_track;
-		JammoEditingTrack* recorded_track;
-
-
-		if (sequencer) {
-			jammo_sequencer_stop(sequencer);
-			g_object_unref(sequencer);
-		}
-		sequencer = jammo_sequencer_new();
-
-		backing_track = jammo_editing_track_new();
-		jammo_sequencer_add_track(sequencer, JAMMO_TRACK(backing_track));
-
-		gchar *filename = g_strdup_printf("%s/songs/%s/comping%s.ogg", DATA_DIR,chum_get_current_song(),chum_get_selected_language());
-		int fd = open(filename,O_RDONLY);
-		if (fd == -1) //File not found
-			{
-			filename = g_strdup_printf("%s/songs/%s/comping.ogg", DATA_DIR,chum_get_current_song()); //use without language postfix
-			}
-		sample1 = jammo_sample_new_from_file(filename);
-		jammo_editing_track_add_sample(backing_track, sample1, 0);
-		g_free(filename);
-
-
-		//for recorded singing
-		recorded_track = jammo_editing_track_new(); 
-		jammo_sequencer_add_track(sequencer, JAMMO_TRACK(recorded_track));
-		gchar* filename2 = g_strdup_printf("%s/last_recorded.wav",JAMMO_DIRECTORY);
-		sample2 = jammo_sample_new_from_file(filename2);
-		g_free(filename2);
-
-		jammo_editing_track_add_sample(recorded_track, sample2, 0);
-		chum_set_state_of_chum(SCREEN_PLAY_RECORDED);
-
-		jammo_mentor_speak_with_callback(jammo_mentor_get_default(), "/opt/jammo/mentor_speech/mentor_welldone_fi.ogg", singingame_start_sequencer, NULL);
-		}
-
-	else if (chum_get_state_of_chum()==SCREEN_PLAY_RECORDED	)
-		{
-		printf("chum_get_state_of_chum()==SCREEN_PLAY_RECORDED\n");
-		jammo_mentor_shut_up(jammo_mentor_get_default());
-		jammo_sequencer_stop(sequencer);
-
-		//'actor' is pressed button, and it's parent is song-*-view
-		//Hide it
-		ClutterActor* parent = clutter_actor_get_parent(CLUTTER_ACTOR(actor));
-		clutter_actor_hide(parent);
-
-		//And go back to songselection
-		ClutterActor* view = jammo_get_actor_by_id("songselection-view");
-		clutter_actor_show(view);
-
-		//And show mentor again
-		tangle_actor_show(TANGLE_ACTOR(jammo_mentor_get_default()));
-
-
-
-		//Automatic saving. THIS MUST BE DOCUMENTED.
-
-		//Saving to cupboard and to log.
-		gchar* cmd;
-
-
-		//Copy that recorded song to 'singings' and name it.
-		cmd = g_strdup_printf("cp %s/last_recorded.wav %s/%s.wav", JAMMO_DIRECTORY, SINGINGS_DIRECTORY,chum_get_current_song() );
-		if (system(cmd)) 
-			printf("Error, can't call '%s'\n",cmd);
-
-
-		//And make it permanent copy for carer/researcher. Filenames look like this: log_singing_2010.02.22_21.16.27.wav
-		cmd = g_strdup_printf("cp %s/last_recorded.wav %s/log_singing_`date +%%Y.%%m.%%d_%%H.%%M.%%S`.wav",JAMMO_DIRECTORY,JAMMO_DIRECTORY);
-		if (system(cmd)) 
-			printf("Error, can't call '%s'\n",cmd);
-
-		g_free(cmd);
-		}
-	else if (chum_get_state_of_chum()==SCREEN_FROM_CUPBOARD	)
-		{
-		printf("chum_get_state_of_chum()==SCREEN_FROM_CUPBOARD\n");
-
-		//Show mentor again
-		tangle_actor_show(TANGLE_ACTOR(jammo_mentor_get_default()));
-
-		//Hide all other views
-		ClutterActor* mainview;
-		mainview = jammo_get_actor_by_id("main-views-widget"); //No drawing on stage anymore, use this.
-		clutter_container_foreach(CLUTTER_CONTAINER(mainview), tangle_actor_hide_animated, NULL);
-
-		//Go to cupboard
-		gchar *new_name = g_strdup_printf("cupboard-view");
-		ClutterActor* new_view;
-		new_view = jammo_get_actor_by_id(new_name);	
-		clutter_actor_show(new_view);
-		}
-
-	else printf("Singinggame_cross_click: Error\n");
+	//This is essential. If user press cross when mentor is talking, this sequencer_stop is handled _before_ sequencer_play
+	jammo_mentor_shut_up(jammo_mentor_get_default());
+	
+	jammo_sequencer_stop(singinggame_sequencer);
 }
 
 
-static void do_hiding_and_dimming(){
+void singinggame_start_songselection(){
 	//Add transperency to songs which do not have current language vocal-track.
 	//Hide advanced songs, when game level is easy.
 	///////////////////////////////////////////////////////////////////////////////
@@ -430,10 +414,18 @@ static void do_hiding_and_dimming(){
 					//printf("  show this\n");
 					clutter_actor_show(button);
 					}
+				
+			
 					
 			}
 	}
 	
+	//Tune size of container
+	if (chum_is_easy_game())
+		clutter_actor_set_size(container, 670.0, 480); //Two rows
+	else
+		clutter_actor_set_size(container, 670.0, 600);  //Three rows
+
 	g_list_free(songlist);
 }
 
@@ -449,7 +441,7 @@ void singinggame_duetto_clicked(TangleAction *action, GObject *source, const gch
 
 	gchar *filepath1;
 	filepath1 =  g_strdup_printf("/opt/jammo/mentor_speech/mentor_%s_selected_fi.ogg", name);
-	printf("pass to mentor: '%s' \n",filepath1);
+	//printf("pass to mentor: '%s' \n",filepath1);
 
 	jammo_mentor_speak(jammo_mentor_get_default(), filepath1);
 
@@ -462,34 +454,23 @@ void singinggame_duetto_clicked(TangleAction *action, GObject *source, const gch
 	texture = jammo_get_actor_by_id(new_name);
 	tangle_button_set_normal_background_actor(TANGLE_BUTTON(button), texture);
 	
-	do_hiding_and_dimming();
 }
 
 
-void singingame_show_songselection(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
-		ClutterActor* view;
-		if ((view = jammo_get_actor_by_id("songselection-view"))) {
-			clutter_actor_show(view);
-		}
-		
-		if ((view = jammo_get_actor_by_id("main-menu-view"))) {
-			clutter_actor_hide(view);
-		}
-		do_hiding_and_dimming();
-}
+
 
 
 void singinggame_language_clicked(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
 	ClutterActor* ct =CLUTTER_ACTOR(source);
 	const gchar* name = clutter_actor_get_name(ct);
-	printf("Lan clicked. get_name: '%s'\n",name);
+	printf("Language clicked. get_name: '%s'\n",name);
 
 	chum_set_selected_language(name);
 
 
 	gchar *filepath1;
 	filepath1 =  g_strdup_printf("/opt/jammo/mentor_speech/mentor%s_selected_fi.ogg", name);
-	printf("pass to mentor: '%s' \n",filepath1);
+	//printf("pass to mentor: '%s' \n",filepath1);
 
 	jammo_mentor_speak(jammo_mentor_get_default(), filepath1);
 
@@ -508,7 +489,6 @@ void singinggame_language_clicked(TangleAction *action, GObject *source, const g
 		clutter_actor_hide(view);
 	}
 
-	do_hiding_and_dimming();
 }
 
 
