@@ -11,7 +11,11 @@
 
 static JammoSequencer* composing_sequencer;
 static JammoTrackView* bottom_track_view;
+static JammoTrackView* upper_track_view; //this is not used in 'easy' mode
 
+static ClutterActor* view_to_go_when_end; //After start this points themeselection or cupboard
+
+static gchar* prefix_for_saving; //This will contain theme-name, -variation and easy/advanced-mode
 
 /*
 Saves only bottom track_view. TODO
@@ -22,7 +26,8 @@ static gboolean save_composition() {
 	gchar *outputFilename;
 
 	// TODO: use date and timestamp for filenames
-	outputFilename = g_strdup_printf("%s/composition.txt", COMPOSITIONS_DIRECTORY);
+	//outputFilename = g_strdup_printf("%s/%s.txt", COMPOSITIONS_DIRECTORY,prefix_for_saving);
+	outputFilename = g_strdup_printf("%s/last_composition.txt", JAMMO_DIRECTORY);
 
 	ofp = fopen(outputFilename, "w");
 	g_free(outputFilename);
@@ -53,20 +58,52 @@ static gboolean save_composition() {
 		fprintf(ofp,"%s\n",image);
 		}
 
+	if (upper_track_view!= NULL) 
+		{
+		fprintf(ofp,"-1\n#\n#\n");
+		track_view = upper_track_view;
+		g_object_get(track_view,"slot-size",&slot_size,NULL);
+		
+		widget = TANGLE_WIDGET(tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(track_view)));
+		for (children = tangle_widget_get_children_readonly(widget); children; children = children->next)
+			{
+			jsb = JAMMO_SAMPLE_BUTTON(children->data);
+			//printf("x: '%lf' \n",clutter_actor_get_x(CLUTTER_ACTOR(jsb)));
+			printf("slot: '%lf' \n",clutter_actor_get_x(CLUTTER_ACTOR(jsb))/slot_size);
+			fprintf(ofp,"%lf\n",clutter_actor_get_x(CLUTTER_ACTOR(jsb))/slot_size);
+			gchar* sound;
+			gchar* image;
+			g_object_get(jsb, "sample-filename", &sound, NULL);
+			g_object_get(jsb, "image-filename", &image, NULL);
+			printf("image: '%s'. sound: '%s' \n",image,sound);
+			fprintf(ofp,"%s\n",sound);
+			fprintf(ofp,"%s\n",image);
+			}
+		}
+
 	fclose(ofp);
+	
+	//Saving to cupboard 
+	gchar* cmd;
+
+	// Filenames look like this: _2010.02.22_21.16.27.wav
+	cmd = g_strdup_printf("cp %s/last_composition.txt %s/%s-`date +%%Y.%%m.%%d_%%H.%%M.%%S`.txt",JAMMO_DIRECTORY,COMPOSITIONS_DIRECTORY,prefix_for_saving);
+	if (system(cmd)) 
+		printf("Error, can't call '%s'\n",cmd);
+
+	g_free(cmd);
+	
 	return FALSE;
 }
 
 
-static void close_themeselection(gpointer data){
-	ClutterActor* new_view;
-	new_view = jammo_get_actor_by_id("themeselection-view");	
 
+static void close_themeselection(gpointer data){
 	TangleView* view = TANGLE_VIEW(data);
 
 	tangle_actor_destroy_animated(TANGLE_ACTOR(view));
 	//tangle_actor_hide_animated(TANGLE_ACTOR(view));
-	clutter_actor_show(new_view);
+	clutter_actor_show(view_to_go_when_end);
 	
 }
 
@@ -162,7 +199,7 @@ return sin (phasing(clutter_timeline_get_progress (clutter_alpha_get_timeline (a
 }
 
 
-
+static ClutterScore* score; /*This is container for animation timelines of samplebuttons*/
 
 /*
 This function creates new jammo_sample_button from given parameters and put it to given parent ClutterActor.
@@ -176,22 +213,28 @@ static ClutterActor* create_samplebutton(gchar* themename, gchar* variation, gch
 	//printf("sound file: '%s'\n",full_sound);
 	sample_button = jammo_sample_button_new_from_files(full_image, full_sound);
 
+
 	//Each animation has own phase
 	int phase = (rand () % 100) + 1;
-	printf("phase %d\n",phase);
+	//printf("phase %d\n",phase);
 	
 	//Each animation has own wobbling 'mode'-function
 	gulong sine_function;
 	sine_function = clutter_alpha_register_func (sine_wave, GINT_TO_POINTER(phase));
-	
+
+	ClutterTimeline *timeline1 = tangle_timeline_new(5000+phase*20);  //each has own rhythm (use same random number than phase)
+	clutter_score_append( score, NULL, timeline1);
+
 	ClutterAnimation* animation;
-	animation = clutter_actor_animate (sample_button, 
-            sine_function, 5000+phase*20, //each has own rhythm (we can use same random number, because phase is not visible on user in anyway. 
-            "opacity", 100,  //do not make it too transparency (0=transparency, 255=opaque)
-            "scale-y", 0.9,  //add some vertical scaling, this looks movement, but doesn't 
+	animation = clutter_actor_animate_with_timeline (sample_button, 
+            sine_function, 
+            timeline1,
+            "opacity", 150,  //do not make it too transparency (0=transparency, 255=opaque)
+            "scale-y", 0.9,  //add some vertical scaling, this looks movement, but doesn't disturb movement
             NULL);
 	clutter_animation_set_loop (animation,TRUE);
-	
+	g_object_unref(timeline1);
+
 	g_free(full_image);
 	g_free(full_sound);
 	clutter_actor_set_position(sample_button, x, y);
@@ -224,50 +267,57 @@ static void menu_clicked(ClutterTexture *actor, ClutterEvent *event, gpointer us
 		tangle_actor_show(TANGLE_ACTOR(user_data));
 	}
 }
-static gboolean on_play_button_clicked(ClutterActor* play_button, ClutterEvent *event, gpointer user_data);
 
-static gboolean on_stop_button_clicked(ClutterActor* play_button, ClutterEvent *event, gpointer user_data) {
-	printf("Play button pressed (stop)\n");
-	
-	jammo_sequencer_stop(JAMMO_SEQUENCER(user_data));
+
+
+static gboolean on_play_button_clicked(TangleButton* tanglebutton, gpointer sequencer){
+	const gchar* name = clutter_actor_get_name (CLUTTER_ACTOR(tanglebutton));
+
+	if (strncmp(name,"play",4)==0)
+		{
+			//clutter_score_stop (score); //This doesn't stop timelines inside score
+			GSList *timeline;
+			for (timeline = clutter_score_list_timelines(score);timeline;timeline = timeline->next)
+				{
+				clutter_timeline_pause(CLUTTER_TIMELINE(timeline->data));
+				}
+		printf("Play button pressed (play)\n");
+		tangle_actor_hide_animated(TANGLE_ACTOR(jammo_mentor_get_default()));
+
+		jammo_sequencer_play(JAMMO_SEQUENCER(sequencer));
+
+		tangle_button_set_normal_background_actor(tanglebutton,clutter_texture_new_from_file("/opt/jammo/buttons/stop-button.png", NULL));
+		tangle_button_set_interactive_background_actor(tanglebutton,clutter_texture_new_from_file("/opt/jammo/buttons/stop-button.png", NULL));
+		clutter_actor_set_name(CLUTTER_ACTOR(tanglebutton),"stop");
+		}
+
+	else if (strncmp(name,"stop",4)==0)
+		{
+		printf("Play button pressed (stop)\n");
+		jammo_sequencer_stop(JAMMO_SEQUENCER(sequencer)); //This will automatically cause 'on_sequencer_stopped'
+		}
+
+
+	return FALSE;
+}
+
+
+static void on_sequencer_stopped(JammoSequencer* sequencer, gpointer play_button) {
+	printf("Sequencer stopped\n");
 	tangle_actor_show(TANGLE_ACTOR(jammo_mentor_get_default()));
+	clutter_score_start (score); //This starts all animations
 
 	//Change button to play
-	ClutterActor* view = clutter_actor_get_parent(play_button);
-	clutter_actor_destroy(play_button);
-	play_button=clutter_texture_new_from_file("/opt/jammo/buttons/play-button.png", NULL);
-	g_object_set(play_button, "x", 0.0,  "y", 195.0, NULL);
-	clutter_container_add(CLUTTER_CONTAINER(view), play_button, NULL);
-	clutter_actor_set_reactive(play_button,TRUE);
-	g_signal_connect (play_button, "button-press-event", G_CALLBACK (on_play_button_clicked),user_data);
-
-	return FALSE;
+	tangle_button_set_normal_background_actor(TANGLE_BUTTON(play_button),clutter_texture_new_from_file("/opt/jammo/buttons/play-button.png", NULL));
+	tangle_button_set_interactive_background_actor(TANGLE_BUTTON(play_button),clutter_texture_new_from_file("/opt/jammo/buttons/play-button.png", NULL));
+	clutter_actor_set_name(CLUTTER_ACTOR(play_button),"play");
 }
 
 
-static gboolean on_play_button_clicked(ClutterActor* play_button, ClutterEvent *event, gpointer user_data) {
-	printf("Play button pressed (play)\n");
-	tangle_actor_hide_animated(TANGLE_ACTOR(jammo_mentor_get_default()));
+gboolean themeselection_load(gpointer data) {
+	gchar* inputFilename = (gchar*) data;
+	printf("Themeselection: Loading composition in file '%s'\n",inputFilename);
 	
-	
-	jammo_sequencer_play(JAMMO_SEQUENCER(user_data));
-
-	//Change button to stop
-	ClutterActor* view = clutter_actor_get_parent(play_button);
-	clutter_actor_destroy(play_button);
-	play_button=clutter_texture_new_from_file("/opt/jammo/buttons/stop-button.png", NULL);
-	g_object_set(play_button, "x", 0.0,  "y", 195.0, NULL);
-	clutter_container_add(CLUTTER_CONTAINER(view), play_button, NULL);
-	clutter_actor_set_reactive(play_button,TRUE);
-	g_signal_connect (play_button, "button-press-event", G_CALLBACK (on_stop_button_clicked),user_data);
-
-	return FALSE;
-}
-
-
-
-gboolean themeselection_load(gchar* inputFilename) {
-	printf("Load button pressed\n");
 	JammoTrackView* track_view = bottom_track_view;
 	FILE *ifp;
 
@@ -324,12 +374,23 @@ gboolean themeselection_load(gchar* inputFilename) {
 
 		printf("name of pngfile='%s'\n",png_name);
 
+		if (slot==-1 && strncmp(png_name,"#",1)==0 && strncmp(wav_name,"#",1)==0 ) //this means 'next-track'
+			{
+			printf("next track\n");
+			if (track_view == upper_track_view) //Somebody tries use three tracks! (Not planned to support this)
+				return FALSE;
+			if (upper_track_view == NULL) //Somebody tries load 'two tracks'-file on easy-mode
+				return FALSE;
+			track_view = upper_track_view;
+			}
 
-		//Make sample_button and put it into track.
-		ClutterActor* sample_button;
-		sample_button = jammo_sample_button_new_from_files(png_name, wav_name);
-		jammo_track_view_add_jammo_sample_button(JAMMO_TRACK_VIEW(track_view), JAMMO_SAMPLE_BUTTON(sample_button), slot);
-
+		else  
+			{
+			//Make sample_button and put it into track.
+			ClutterActor* sample_button;
+			sample_button = jammo_sample_button_new_from_files(png_name, wav_name);
+			jammo_track_view_add_jammo_sample_button(JAMMO_TRACK_VIEW(track_view), JAMMO_SAMPLE_BUTTON(sample_button), slot);
+			}
 
 	}
 	return FALSE;
@@ -396,14 +457,18 @@ static ClutterActor* create_soundfamily_button(const gchar* theme_name, const gc
 }
 
 #define WIDTH_OF_SLOT 40.0
-void themeselection_start_theme(const gchar* name){
+void themeselection_start_theme(const gchar* name, int variation, ClutterActor* where_to_go_after){
+	view_to_go_when_end = where_to_go_after; //We set this static ClutterActor, so we know where to go when composition game is quitted.
+	
 	printf("Theme '%s' starting.\n",name);
 
 	GError* error = NULL;
 
-	//subthemes are 1,2,3.
-	int variation = (rand () % 3) + 1;
-	printf("Random variation is '%d'\n",variation);
+
+	
+	prefix_for_saving = g_strdup_printf("+%s+%d+%s",name,variation,chum_is_easy_game()? "e":"a");
+	//example: +fantasy+2+e
+	//printf("prefix is '%s'\n",prefix_for_saving);
 	
 	//Background for this theme
 	gchar *background_image_name = g_strdup_printf("%s/themes/%s/background.png",DATA_DIR,name);
@@ -445,7 +510,8 @@ void themeselection_start_theme(const gchar* name){
 	clutter_actor_set_reactive(back_to_theme_selection,TRUE);
 	g_signal_connect (back_to_theme_selection, "button-press-event", G_CALLBACK (back_to_theme_selection_clicked),view);
 
-
+	score = clutter_score_new();
+	
 	int tempo_table[] = {0,140,110,90};
 	gchar* background_color_code = "some_default";
 	gchar* backing_track_filepath = "no_any_backing_track";
@@ -743,7 +809,8 @@ if (use_two_tracks) y_offset=80;
 	tangle_widget_add(TANGLE_WIDGET(container_for_tracks), track_view, NULL);
 
 	bottom_track_view = JAMMO_TRACK_VIEW(track_view); //This is needed when loading/saving
-
+	upper_track_view = NULL; //This will be NULL, if use_two_tracks is not TRUE.
+	
 	if (use_two_tracks){
 	//Upper track.
 	JammoEditingTrack* editing_track_theme2 = jammo_editing_track_new();
@@ -754,6 +821,8 @@ if (use_two_tracks) y_offset=80;
 	g_object_set(track_view2, "line-every-nth-slot", 2, NULL);
 	clutter_actor_set_position(track_view2, 0.0, 0.0);
 	tangle_widget_add(TANGLE_WIDGET(container_for_tracks), track_view2, NULL);
+	
+	upper_track_view = JAMMO_TRACK_VIEW(track_view2); //This is needed when loading/saving
 	}
 
 	//Scrolling area for track (or tracks)
@@ -765,14 +834,17 @@ if (use_two_tracks) y_offset=80;
 
 
 
+
 	//Play-button
 	ClutterActor *play_button;
-	play_button=clutter_texture_new_from_file("/opt/jammo/buttons/play-button.png", NULL);
+	play_button = tangle_button_new_with_background_actors(clutter_texture_new_from_file("/opt/jammo/buttons/play-button.png", NULL), clutter_texture_new_from_file("/opt/jammo/buttons/play-button.png", NULL));
 	g_object_set(play_button, "x", 0.0,  "y", 195.0, NULL);
 	clutter_container_add(CLUTTER_CONTAINER(view), play_button, NULL);
 	clutter_actor_set_reactive(play_button,TRUE);
-	g_signal_connect (play_button, "button-press-event", G_CALLBACK (on_play_button_clicked),sequencer);
+	g_signal_connect (play_button, "clicked", G_CALLBACK (on_play_button_clicked),sequencer);
+	clutter_actor_set_name (play_button,"play");
 	
+	g_signal_connect(sequencer, "stopped", G_CALLBACK(on_sequencer_stopped), play_button);
 	/*
 	//Load-button DEBUG
 	ClutterActor *load_button;
@@ -791,5 +863,10 @@ Selected theme is 'name'-property of clicked button.
 */
 void themeselection_theme_clicked(TangleAction *action, GObject *source, const gchar *trigger, TangleProperties *properties){
 	const gchar* name = clutter_actor_get_name(CLUTTER_ACTOR(source));
-	themeselection_start_theme(name);
+	
+	//subthemes are 1,2,3.
+	int variation = (rand () % 3) + 1;
+	printf("Random variation is '%d'\n",variation);
+	
+	themeselection_start_theme(name,variation,jammo_get_actor_by_id("themeselection-view"));
 }
